package de.tu_darmstadt.tk.SmartHomeNetworkSim.control;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.LinkedList;

import javax.tools.JavaCompiler;
import javax.tools.ToolProvider;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Model;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Protocol;

/**
 * Controller which manages Imported Classes of the 
 *
 * @author Andreas T. Meyer-Berg
 */
public class ImportController {
	Model model;
	Controller controller;
	
	/**
	 * Creates a new ImportController
	 * @param model model to be edited
	 * @param controller parent controller
	 */
	public ImportController(Model model, Controller controller) {
		this.model = model;
		this.controller = controller;
	}
	
	/**
	 * Returns the available Protocols of the model
	 * 
	 * @return available protocols
	 */
	public LinkedList<Class<? extends Protocol>> getProtocols(){
		return model.getProtocols();
	}
	
	/**
	 * Adds new Protocol to the model
	 * @param protocol protocol to be added
	 * @return true if it was added
	 */
	public boolean addProtocol(Class<? extends Protocol> protocol){
		if(isValidProtocol(protocol))
			model.addProtocol(protocol);
		else 
			return false;
		return true;
		
	}
	
	/**
	 * Removes protocol from the model
	 * @param protocol protocol to be removed
	 */
	public void removeProtocol(Class<? extends Protocol> protocol){
		model.removeProtocol(protocol);
	}
	
	/**
	 * Returns true if it is a Valid Protocol, false if not
	 * 
	 * @param protocol protocol to be checked
	 * @return true if it is a valid protocol
	 */
	public boolean isValidProtocol(Class<? extends Protocol> protocol){
		try {
			Protocol p = protocol.newInstance();
			//Empty constructor required, to create new instance
			if(p == null)
				throw new Exception("Protocol required an empty constructor");
			// Name shall not be null or empty string
			if(p.getName() == null || p.getName()=="")
				throw new Exception("Protocol name shall not be null or empty string.");
			if(p.getRoles()==null||p.getRoles().length==0)
				throw new Exception("Roles shall not be null or empty");
			// Number of roles have to match
			if(p.getNumberOfRoles()!=p.getRoles().length) 
				throw new Exception("getNumberOfRoles() does not match getRoles().length");
		} catch (Exception e) {
			return false;
		}
		return true;
	}
	
	public Class importJavaClass(File javaFile){
		// Compile
		JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
		compiler.run(null, null, null, javaFile.getPath());

		// Load compiled class. - not working
		try{
		URLClassLoader classLoader = URLClassLoader.newInstance(new URL[] { new File("src/").toURI().toURL() });
		String className = javaFile.getName().substring(javaFile.getName().lastIndexOf('.'));
		Class<?> cls = Class.forName(className, true, classLoader); // Should load the class
		
		return cls;
		}catch(Exception e){
			
		}
		return null;
	}
}
